<?php

declare(strict_types=1);

namespace Tests\Feature\Repository;

use Override;
use Tests\TestCase;
use App\Entity\DrinkType;
use App\Repository\DrinkTypeRepository;

class DrinkTypeRepositoryTest extends TestCase
{
    private DrinkTypeRepository $repository;

    #[Override]
    public function setUp(): void
    {
        parent::setUp();
        $this->setUpDB();
        $this->repository = $this->container->get(DrinkTypeRepository::class);
    }

    public function testFindAll(): void
    {
        // Initially the repository should be empty
        $this->assertCount(0, $this->repository->findAll());

        // Create some drink types
        $drinkType1 = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $drinkType2 = new DrinkType('Fanta', 'Orange soda', 5);

        // Save them to the repository
        $this->repository->save($drinkType1);
        $this->repository->save($drinkType2);

        // Now findAll should return both drink types
        $drinkTypes = $this->repository->findAll();
        $this->assertCount(2, $drinkTypes);
        $this->assertContainsOnlyInstancesOf(DrinkType::class, $drinkTypes);
    }

    public function testFind(): void
    {
        // Create a drink type
        $drinkType = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $this->repository->save($drinkType);

        // Get the ID
        $id = $drinkType->getId();
        $this->assertNotNull($id);

        // Find by ID
        $foundDrinkType = $this->repository->find($id);
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Cola', $foundDrinkType->getName());
        $this->assertEquals('Refreshing cola drink', $foundDrinkType->getDescription());
        $this->assertEquals(10, $foundDrinkType->getDesiredStock());

        // Try to find a non-existent ID
        $nonExistentId = 9999;
        $this->assertNull($this->repository->find($nonExistentId));
    }

    public function testFindOneBy(): void
    {
        // Create a drink type
        $drinkType = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $this->repository->save($drinkType);

        // Find by name
        $foundDrinkType = $this->repository->findOneBy(['name' => 'Cola']);
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Cola', $foundDrinkType->getName());

        // Try to find a non-existent name
        $this->assertNull($this->repository->findOneBy(['name' => 'NonExistent']));
    }

    public function testSave(): void
    {
        // Create a drink type
        $drinkType = new DrinkType('Cola', 'Refreshing cola drink', 10);

        // Save it
        $this->repository->save($drinkType);

        // Check that it was saved
        $id = $drinkType->getId();
        $this->assertNotNull($id);

        // Find it by ID to confirm it's in the database
        $foundDrinkType = $this->repository->find($id);
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Cola', $foundDrinkType->getName());

        // Update it
        $drinkType->setName('Pepsi');
        $this->repository->save($drinkType);

        // Find it again to confirm the update
        $foundDrinkType = $this->repository->find($id);
        $this->assertEquals('Pepsi', $foundDrinkType->getName());
    }

    public function testRemove(): void
    {
        // Create a drink type
        $drinkType = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $this->repository->save($drinkType);

        // Get the ID
        $id = $drinkType->getId();
        $this->assertNotNull($id);

        // Remove it
        $this->repository->remove($drinkType);

        // Try to find it by ID to confirm it's gone
        $this->assertNull($this->repository->find($id));
    }
}
