<?php

declare(strict_types=1);

namespace Tests\Feature\Repository;

use Override;
use Tests\TestCase;
use App\Entity\DrinkType;
use App\Entity\InventoryRecord;
use App\Repository\DrinkTypeRepository;
use App\Repository\InventoryRecordRepository;
use DateTimeImmutable;

class InventoryRecordRepositoryTest extends TestCase
{
    private InventoryRecordRepository $repository;
    private DrinkTypeRepository $drinkTypeRepository;
    private DrinkType $drinkType;

    #[Override]
    public function setUp(): void
    {
        parent::setUp();
        $this->setUpDB();
        $this->repository = $this->container->get(InventoryRecordRepository::class);
        $this->drinkTypeRepository = $this->container->get(DrinkTypeRepository::class);

        // Create a drink type for testing
        $this->drinkType = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $this->drinkTypeRepository->save($this->drinkType);
    }

    public function testFindAll(): void
    {
        // Initially the repository should be empty
        $this->assertCount(0, $this->repository->findAll());

        // Create some inventory records
        $record1 = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));
        $record2 = new InventoryRecord($this->drinkType, 10, new DateTimeImmutable('2023-01-02'));

        // Save them to the repository
        $this->repository->save($record1);
        $this->repository->save($record2);

        // Now findAll should return both records
        $records = $this->repository->findAll();
        $this->assertCount(2, $records);
        $this->assertContainsOnlyInstancesOf(InventoryRecord::class, $records);
    }

    public function testFind(): void
    {
        // Create an inventory record
        $record = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));
        $this->repository->save($record);

        // Get the ID
        $id = $record->getId();
        $this->assertNotNull($id);

        // Find by ID
        $foundRecord = $this->repository->find($id);
        $this->assertInstanceOf(InventoryRecord::class, $foundRecord);
        $this->assertEquals(5, $foundRecord->getQuantity());
        $this->assertEquals('2023-01-01', $foundRecord->getTimestamp()->format('Y-m-d'));

        // Try to find a non-existent ID
        $nonExistentId = 9999;
        $this->assertNull($this->repository->find($nonExistentId));
    }

    public function testFindByDrinkType(): void
    {
        // Create another drink type
        $anotherDrinkType = new DrinkType('Fanta', 'Orange soda', 5);
        $this->drinkTypeRepository->save($anotherDrinkType);

        // Create inventory records for both drink types
        $record1 = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));
        $record2 = new InventoryRecord($this->drinkType, 10, new DateTimeImmutable('2023-01-02'));
        $record3 = new InventoryRecord($anotherDrinkType, 15, new DateTimeImmutable('2023-01-03'));

        // Save them to the repository
        $this->repository->save($record1);
        $this->repository->save($record2);
        $this->repository->save($record3);

        // Find by drink type
        $records = $this->repository->findByDrinkType($this->drinkType);
        $this->assertCount(2, $records);
        $this->assertContainsOnlyInstancesOf(InventoryRecord::class, $records);

        // Check that the records are for the correct drink type
        foreach ($records as $record) {
            $this->assertEquals($this->drinkType->getId(), $record->getDrinkType()->getId());
        }

        // Find by the other drink type
        $otherRecords = $this->repository->findByDrinkType($anotherDrinkType);
        $this->assertCount(1, $otherRecords);
        $this->assertEquals($anotherDrinkType->getId(), $otherRecords[0]->getDrinkType()->getId());
    }

    public function testFindLatestByDrinkType(): void
    {
        // Create inventory records with different timestamps
        $record1 = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));
        $record2 = new InventoryRecord($this->drinkType, 10, new DateTimeImmutable('2023-01-02'));
        $record3 = new InventoryRecord($this->drinkType, 15, new DateTimeImmutable('2023-01-03'));

        // Save them to the repository
        $this->repository->save($record1);
        $this->repository->save($record2);
        $this->repository->save($record3);

        // Find the latest record
        $latestRecord = $this->repository->findLatestByDrinkType($this->drinkType);
        $this->assertInstanceOf(InventoryRecord::class, $latestRecord);
        $this->assertEquals(15, $latestRecord->getQuantity());
        $this->assertEquals('2023-01-03', $latestRecord->getTimestamp()->format('Y-m-d'));
    }

    public function testFindByTimestampRange(): void
    {
        // Create inventory records with different timestamps
        $record1 = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));
        $record2 = new InventoryRecord($this->drinkType, 10, new DateTimeImmutable('2023-01-02'));
        $record3 = new InventoryRecord($this->drinkType, 15, new DateTimeImmutable('2023-01-03'));
        $record4 = new InventoryRecord($this->drinkType, 20, new DateTimeImmutable('2023-01-04'));
        $record5 = new InventoryRecord($this->drinkType, 25, new DateTimeImmutable('2023-01-05'));

        // Save them to the repository
        $this->repository->save($record1);
        $this->repository->save($record2);
        $this->repository->save($record3);
        $this->repository->save($record4);
        $this->repository->save($record5);

        // Find records in a specific range
        $start = new DateTimeImmutable('2023-01-02');
        $end = new DateTimeImmutable('2023-01-04');
        $records = $this->repository->findByTimestampRange($start, $end);

        $this->assertCount(3, $records);

        // Check that the records are within the range
        foreach ($records as $record) {
            $timestamp = $record->getTimestamp();
            $this->assertTrue($timestamp >= $start && $timestamp <= $end);
        }
    }

    public function testSave(): void
    {
        // Create an inventory record
        $record = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));

        // Save it
        $this->repository->save($record);

        // Check that it was saved
        $id = $record->getId();
        $this->assertNotNull($id);

        // Find it by ID to confirm it's in the database
        $foundRecord = $this->repository->find($id);
        $this->assertInstanceOf(InventoryRecord::class, $foundRecord);
        $this->assertEquals(5, $foundRecord->getQuantity());

        // Update it
        $record->setQuantity(10);
        $this->repository->save($record);

        // Find it again to confirm the update
        $foundRecord = $this->repository->find($id);
        $this->assertEquals(10, $foundRecord->getQuantity());
    }

    public function testRemove(): void
    {
        // Create an inventory record
        $record = new InventoryRecord($this->drinkType, 5, new DateTimeImmutable('2023-01-01'));
        $this->repository->save($record);

        // Get the ID
        $id = $record->getId();
        $this->assertNotNull($id);

        // Remove it
        $this->repository->remove($record);

        // Try to find it by ID to confirm it's gone
        $this->assertNull($this->repository->find($id));
    }
}
