<?php

declare(strict_types=1);

namespace Tests\Feature\Service;

use Override;
use Tests\TestCase;
use App\Entity\DrinkType;
use App\Service\DrinkTypeService;
use App\Repository\DrinkTypeRepository;
use InvalidArgumentException;

class DrinkTypeServiceTest extends TestCase
{
    private DrinkTypeService $service;
    private DrinkTypeRepository $repository;

    #[Override]
    public function setUp(): void
    {
        parent::setUp();
        $this->setUpDB();
        $this->repository = $this->container->get(DrinkTypeRepository::class);
        $this->service = $this->container->get(DrinkTypeService::class);
    }

    public function testGetAllDrinkTypes(): void
    {
        // Initially the repository should be empty
        $this->assertCount(0, $this->service->getAllDrinkTypes());

        // Create some drink types
        $drinkType1 = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $drinkType2 = new DrinkType('Fanta', 'Orange soda', 5);
        $this->repository->save($drinkType1);
        $this->repository->save($drinkType2);

        // Now getAllDrinkTypes should return both drink types
        $drinkTypes = $this->service->getAllDrinkTypes();
        $this->assertCount(2, $drinkTypes);
        $this->assertContainsOnlyInstancesOf(DrinkType::class, $drinkTypes);
    }

    public function testGetDrinkTypeById(): void
    {
        // Create a drink type
        $drinkType = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $this->repository->save($drinkType);

        // Get the ID
        $id = $drinkType->getId();
        $this->assertNotNull($id);

        // Find by ID
        $foundDrinkType = $this->service->getDrinkTypeById($id);
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Cola', $foundDrinkType->getName());
        $this->assertEquals('Refreshing cola drink', $foundDrinkType->getDescription());
        $this->assertEquals(10, $foundDrinkType->getDesiredStock());

        // Try to find a non-existent ID
        $nonExistentId = 9999;
        $this->assertNull($this->service->getDrinkTypeById($nonExistentId));
    }

    public function testGetDrinkTypeByName(): void
    {
        // Create some drink types
        $drinkType1 = new DrinkType('Cola', 'Refreshing cola drink', 10);
        $drinkType2 = new DrinkType('Fanta', 'Orange soda', 5);
        $this->repository->save($drinkType1);
        $this->repository->save($drinkType2);

        // Find by name
        $foundDrinkType = $this->service->getDrinkTypeByName('Cola');
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Cola', $foundDrinkType->getName());
        $this->assertEquals('Refreshing cola drink', $foundDrinkType->getDescription());
        $this->assertEquals(10, $foundDrinkType->getDesiredStock());

        // Try to find a non-existent name
        $this->assertNull($this->service->getDrinkTypeByName('nonexistent'));
    }

    public function testCreateDrinkType(): void
    {
        // Create a new drink type
        $drinkType = $this->service->createDrinkType('Cola', 'Refreshing cola drink', 10);

        // Check that it was created correctly
        $this->assertInstanceOf(DrinkType::class, $drinkType);
        $this->assertEquals('Cola', $drinkType->getName());
        $this->assertEquals('Refreshing cola drink', $drinkType->getDescription());
        $this->assertEquals(10, $drinkType->getDesiredStock());

        // Check that it's in the database
        $foundDrinkType = $this->service->getDrinkTypeByName('Cola');
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Cola', $foundDrinkType->getName());
        $this->assertEquals('Refreshing cola drink', $foundDrinkType->getDescription());
        $this->assertEquals(10, $foundDrinkType->getDesiredStock());

        // Try to create a drink type with the same name (should throw an exception)
        $this->expectException(InvalidArgumentException::class);
        $this->service->createDrinkType('Cola', 'Another cola drink', 5);
    }

    public function testUpdateDrinkType(): void
    {
        // Create a drink type
        $drinkType = $this->service->createDrinkType('Cola', 'Refreshing cola drink', 10);

        // Update the description
        $updatedDrinkType = $this->service->updateDrinkType($drinkType, null, 'Updated description', null);
        $this->assertEquals('Cola', $updatedDrinkType->getName());
        $this->assertEquals('Updated description', $updatedDrinkType->getDescription());
        $this->assertEquals(10, $updatedDrinkType->getDesiredStock());

        // Check that it was updated in the database
        $foundDrinkType = $this->service->getDrinkTypeByName('Cola');
        $this->assertEquals('Updated description', $foundDrinkType->getDescription());

        // Update the desired stock
        $updatedDrinkType = $this->service->updateDrinkType($drinkType, null, null, 20);
        $this->assertEquals('Cola', $updatedDrinkType->getName());
        $this->assertEquals('Updated description', $updatedDrinkType->getDescription());
        $this->assertEquals(20, $updatedDrinkType->getDesiredStock());

        // Check that it was updated in the database
        $foundDrinkType = $this->service->getDrinkTypeByName('Cola');
        $this->assertEquals(20, $foundDrinkType->getDesiredStock());

        // Update the name
        $updatedDrinkType = $this->service->updateDrinkType($drinkType, 'Pepsi', null, null);
        $this->assertEquals('Pepsi', $updatedDrinkType->getName());
        $this->assertEquals('Updated description', $updatedDrinkType->getDescription());
        $this->assertEquals(20, $updatedDrinkType->getDesiredStock());

        // Check that it was updated in the database
        $foundDrinkType = $this->service->getDrinkTypeByName('Pepsi');
        $this->assertInstanceOf(DrinkType::class, $foundDrinkType);
        $this->assertEquals('Pepsi', $foundDrinkType->getName());
        $this->assertEquals('Updated description', $foundDrinkType->getDescription());
        $this->assertEquals(20, $foundDrinkType->getDesiredStock());

        // Create another drink type
        $drinkType2 = $this->service->createDrinkType('Fanta', 'Orange soda', 5);

        // Try to update the name to an existing name (should throw an exception)
        $this->expectException(InvalidArgumentException::class);
        $this->service->updateDrinkType($drinkType2, 'Pepsi', null, null);
    }

    public function testDeleteDrinkType(): void
    {
        // Create a drink type
        $drinkType = $this->service->createDrinkType('Cola', 'Refreshing cola drink', 10);

        // Check that it exists
        $this->assertInstanceOf(DrinkType::class, $this->service->getDrinkTypeByName('Cola'));

        // Delete it
        $this->service->deleteDrinkType($drinkType);

        // Check that it's gone
        $this->assertNull($this->service->getDrinkTypeByName('Cola'));
    }
}
